// bersetzen:
// moc -i -o edit_moc.h edit.cc
// gcc -I/usr/lib/qt/include/ -L/usr/lib/qt/lib -lqt edit.cc

#include <qapplication.h>        
#include <qmultilinedit.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qstring.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qvbox.h>
#include <qhbox.h>


class Editor : public QVBox        // vertikale Widget-Box
{
  Q_OBJECT
public:
  Editor(QWidget *parent=0);
private:
  QMultiLineEdit *m;               // das Editor-Feld
public slots:
  open();                          // Datei laden
  save();                          // Datei speichern
signals:
  void newfile(const QString &);   // neuer Dateiname
};


Editor::Editor(QWidget *parent)
  : QVBox(parent)
{
  m = new QMultiLineEdit(this);    // Editor-Feld oben
  QHBox *h = new QHBox(this);      // eine horizontale Box

// drei Pushbuttons nebeneinander; Signal clicked() jedes
// Buttons mit dem entsprechenden Slot verbinden:
// o ("ffnen") -> Editor.open()
// s ("speichern") -> Editor.save()
// q ("Ende") -> qApp.quit() moc -i -o edit_moc.h edit.cc
  QPushButton *o = new QPushButton("ffnen", h);
  connect(o, SIGNAL(clicked()), this, SLOT(open()));
  QPushButton *s = new QPushButton("speichern", h);
  connect(s, SIGNAL(clicked()), this, SLOT(save()));
  QPushButton *q = new QPushButton("Ende", h);
  connect(q, SIGNAL(clicked()), qApp, SLOT(quit()));

// Signal newfile() setzt neuen Fenstertitel
  connect(this, SIGNAL(newfile(const QString &)),
          this, SLOT(setCaption(const QString &)));
  emit newfile("Neue Datei");      // noch keine Datei geladen
}  // Editor()


Editor::open()
// Datei ffnen; Slot, der mit Signal clicked() des
// ffnen-Buttons verbunden ist
{
  QString fname = QFileDialog::getOpenFileName();
  if (fname)                       // Datei angewhlt
  {
    QFile f(fname);                // IO-Device fr Datei
    if (f.open(IO_ReadOnly))       // Datei ffnen
    {
      m->clear();                  // Edit-Feld lschen
      QTextStream t(&f);           // zeilenweise via TextStream
      while (!t.eof())             // lesen und an den Inhalt
       m->append(t.readLine());    // des Edit-Feldes anhngen
      emit newfile(fname);         // Dateinamen verschicken
    }
  }
} // open()


Editor::save()
// Datei speichern; Slot, der mit Signal clicked() des
// Speichern-Buttons verbunden ist
{
  QString text = m->text();        // Inhalt des Editfeldes holen
  QString fname = QFileDialog::getSaveFileName();
  if (fname)                       // Dateiname gewhlt
  {
    QFile f(fname);                // IO-Device fr Datei
    if (f.open(IO_WriteOnly))      // Datei ffnen
    {
      QTextStream t(&f);           // Text aus Editfeld via
      t << m->text();              // IO-Stream in die Datei
    }
  }
} // save()


int main(int argc, char *argv[])
{
  QApplication a(argc, argv);   // Anwendung initialisieren
  Editor ed;                    // Haupt-Widget erzeugen ...
  a.setMainWidget(&ed);         // ... setzen ...
  ed.show();                    // ... und anzeigen
  return a.exec();              // fertig
}  // main()

#include "edit_moc.h"

