/* Kompilieren: gcc `gtk-config --cflags --libs` pixmap.c */

#include <gtk/gtk.h>

/* Globale Variablen */

GtkWidget *window;            /* Hauptfenster */
GtkWidget *table;             /* Tabelle zum Anordnen  der Widgets */
GtkWidget *pixmap = NULL;     /* die angezeigte Pixmap */
GtkWidget *fd = NULL;         /* Dateiauswahl-Dialog */


/* Callback-Funktion: Pixmap laden und anzeigen */
/* Wird bei "OK" im Dateiauswahl-Dialog aufgerufen */

void draw_pixmap(GtkWidget *w, GtkFileSelection *fsel)
{
  GdkPixmap *xpm;             /* Pixmap im GDK-Format */
  GdkBitmap *mask;            /* Transparenz-Maske */
  GtkStyle *style;            /* Eigenschaften des Fensters */
  char *fname;                /* Dateiname */
  char *c;

  fname = c = gtk_file_selection_get_filename(fsel);  /* Dateiname holen */
  while (*c > '\0') c++; c--;                   /* Workaround fr Bug in */
  if ('/' == *c) fname = NULL;           /* gdk_pixmap_create_from_xpm() */

  if (fname)                                        /* Datei ausgewhlt? */
  {
    if (pixmap) gtk_widget_hide(pixmap);              /* alte Pixmap weg */

    style = gtk_widget_get_style(window);  /* Eigenschaften des Fensters */
    xpm = gdk_pixmap_create_from_xpm(window->window, &mask,
     &style->bg[GTK_STATE_NORMAL], fname);               /* Pixmap laden */
    if (xpm) pixmap = gtk_pixmap_new(xpm, mask);  /* gtk_pixmap erzeugen */
    else pixmap = NULL;

    if (pixmap)                                       /* pixmap geladen? */
    {                /* in Spalten 0 und 1, Zeile 0 der Tabelle anzeigen */
      gtk_table_attach_defaults(GTK_TABLE(table), pixmap, 0, 2, 0, 1);
      gtk_widget_show(pixmap);
    }
  }   /* if (fname) */
}   /* draw_pixmap() */



/* Callback-Funktion: Dateiauswahl-Dialog zerstren */
/* Wird bei "Cancel" im Dateiauswahl-Dialog aufgerufen */

void destroy_filesel(GtkWidget *w, GtkWidget *filesel)
{
  gtk_widget_destroy(filesel);
  fd = NULL;
} /* destroy_filesel() */


/* Callback-Funktion: Datei ffnen */
/* Wird vom Button "ffnen" aufgerufen */

void open_file()
{
  if (!fd)          /* Dateiauswahl-Dialog ist nicht geffnet: aufmachen */
  {
    fd = gtk_file_selection_new("Pixmap auswhlen");
    
    /* bei "Cancel" oder Schlieen des Fenster via WM: destroy_filesel() */
    gtk_signal_connect(GTK_OBJECT(fd), "destroy",
                       GTK_SIGNAL_FUNC(destroy_filesel), fd);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fd)->cancel_button),
                        "clicked", GTK_SIGNAL_FUNC(destroy_filesel), fd);
    /* bei "OK": draw_pixmap() */
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fd)->ok_button),
                        "clicked", GTK_SIGNAL_FUNC(draw_pixmap), fd);
    gtk_widget_show(fd);                              /* Dialog anzeigen */
  } /* if (!fd) */
} /* open_file() */


/* Tabelle bauen und Buttons anzeigen. */
/* Rckgabe: Das Tabellen-Widget */

GtkWidget* init_table(GtkWidget *win)
{
  GtkWidget *tbl;             /* die Tabelle */
  GtkWidget *quit, *load;     /* Push-Buttons */

  /* 2x2 Tabelle mit unterschiedl. groen Elementen (homogeneous=FALSE) */
  /* und 20 bzw. 10 Pixeln Platz ("Spacing") zwischen den Elementen */
  tbl = gtk_table_new(2, 2, FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(tbl), 20);
  gtk_table_set_col_spacings(GTK_TABLE(tbl), 10);

  /* Tabelle ins Fenster (10 Pixel Rand drumrum) und anzeigen */
  gtk_container_set_border_width (GTK_CONTAINER (window), 10);
  gtk_container_add(GTK_CONTAINER(win), tbl);
  gtk_widget_show(tbl);

  /* 2 Pushbuttons: ffnen (Callback: open_file()) 
  /* und Ende (Callback: gtk_widget_destroy(window)) */
  load = gtk_button_new_with_label("ffnen");
  gtk_signal_connect_object(GTK_OBJECT(load), "clicked", 
   GTK_SIGNAL_FUNC(open_file), GTK_OBJECT(win));
  quit = gtk_button_new_with_label("Beenden");
  gtk_signal_connect_object(GTK_OBJECT(quit), "clicked", 
   GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(win));

  /* ffnen in Spalte 0, Ende in Spalte 1; beide Buttons in Zeile 1 */
  gtk_table_attach_defaults(GTK_TABLE(tbl), load, 0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(tbl), quit, 1, 2, 1, 2);
  
  /* Buttons anzeigen; GtkWidget *tbl zurckgeben */
  gtk_widget_show(load);
  gtk_widget_show(quit);
  return(tbl);
}   /* init_table() */


int main(int argc, char *argv[])
{
  gtk_init (&argc, &argv);
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_realize(window);    /* Fenster-Eigenschaften initialisieren */
  gtk_signal_connect(GTK_OBJECT(window), "destroy", /* destroy Window -> */
                     GTK_SIGNAL_FUNC(gtk_main_quit), NULL); /* Prog.Ende */
  table = init_table(window);
  gtk_widget_show(window);                      /* Hauptfenster anzeigen */
  gtk_main();
  return(0);
}

